

classdef riDevice < handle
    % Resolved Instrument's MATLAB class for controlling the DPD80
    
    properties (SetAccess = private)
        opened = false % (boolean) Indicates whether a connection to a DPD80 is opened.
    end

    properties (Access = private)
        device_pointer = 0 % Private property
    end
    
    methods (Hidden, Static)
        function addlistener
        end
        
        function delete
        end
        
        function eq
        end
        
        function findobj
        end
        
        function findprop
        end
        
        function ge
        end
        
        function gt
        end
        
        
        function le
        end
        
        function lt
        end
        
        function ne
        end
        
        function notify
        end
    end
    
    methods
        function obj = riDevice(serial)
            %RIDEVICE Creates a riDevice instance and connects to a DPD80
            %
            %SYNTAX: device = RIDEVICE(serial)
            %
            %INPUTS:
            %       serial (string): (optional) Serial number of the device to open.
            %
            %OUTPUTS:
            %       device: The instance of the device class.
            %
            if nargin == 0
                obj.device_pointer = open_gw();
            elseif nargin == 1
                obj.device_pointer = open_gw(serial);
            end
            obj.opened = true;

        end
        
        function open(obj, serial)
            %OPEN Opens a connection to a DPD80.
            %
            %SYNTAX: device.open(serial)
            %
            %INPUTS:
            %       serial (string): (optional) Serial number of the device to open.
            %
            if obj.device_pointer == 0;
                if nargin == 0
                    obj.device_pointer = open_gw();
                elseif nargin == 1
                    obj.device_pointer = open_gw(serial);
                end
                obj.opened = true;

            else
                display('The device was already opened');
            end
        end
        
        function data = get_raw_data(obj, nsamples, mask)
            %GET_RAW_DATA Collects data from the riDevice.
            %
            %SYNTAX: data = device.get_raw_data(nsamples, mask)
            %
            %INPUT nsamples (int): Number of samples to collect
            %
            %      mask (boolean): (optional )Removes bits 15 and
            %      16 from the data. These bits correspond to the the Trig
            %      and Sync Inputs. Default is true.
            %
            %OUTPUT data (1-D Matrix, ints): The collected data.
            %
            if obj.device_pointer == 0
                error('Device needs to be opened.')
            end
            
            if nargin == 2
                data = get_raw_data_gw(obj.device_pointer, nsamples);
            elseif nargin == 3
                data = get_raw_data(obj.device_pointer, nsamples, mask);
            end
        end
        
        function r = get_relative_responsivity(obj, wavelength)
            %GET_RELATIVE_RESPONSIVITY Returns your device's relative responsivity at the given wavelength.
            %
            %SYNTAX: r = device.get_relative_responsivity(wavelength)
            %
            %INPUTs:
            %       wavelength (double): A wavelength in nm.
            %
            %OUTPUTS:
            %       r (double): The relative responsivity (between 0 and 1) at the
            %       given wavelength. If the input is outside of the defined
            %       responsivity range, 0 will be returned.
            if obj.device_pointer == 0
                error('Device needs to be opened.')
            end
            
            r = get_rel_responsivity_gw(obj.device_pointer, wavelength);
        end
        
        function speed = get_usb_speed(obj)
            %GET_USB_SPEED Returns the speed of your USB connection
            %
            %SYNTAX: speed = device.get_usb_speed()
            %
            %OUTPUTS:
            %       speed (string): The speed of your USB connection. The value should
            %       be 'super', but if you are using a port or cable not
            %       compatable with USB 3.0, another value may be returned.
            %
            %REMARKS:
            %       Useful for detecting if your compuuter's USB port is
            %       USB 3.0 compatable.
            %
            if obj.device_pointer == 0
                error('Device needs to be opened.')
            end
            
            speed = get_usb_speed_gw(obj.device_pointer);
        end
        
        function port_drive(obj, port, val)
            %PORT_DRIVE Sets a DPD80 port to either 3.3V or 0V.
            %
            %SYNTAX: device.port_drive(port, val)
            %
            %INPUTS:
            %       port (string): The port that you want to set. Acceptable values
            %       are 'A' or 'B'.
            %
            %       val (boolean): true will set the port to 3.3 V,
            %       false will set the port to 0 V.
            %
            %See Also PORT_READ, PORT_PULSE, PORT_PWM
            if obj.device_pointer == 0
                error('Device needs to be opened.')
            end
            
            port_drive_gw(obj.device_pointer, port, val);
            
        end
        
        function port_pulse(obj, port, threshold)
            %PORT_PULSE Pulses a DPD80 port.
            %
            %SYNTAX: device.port_pulse(port, threshold)
            %
            %INPUTS:
            %       port (string): The port that you want to pulse. Acceptable values
            %       are 'A' or 'B'. Please note that port A is low
            %       passed at ~1 KHz.
            %
            %       threshold (int): The number of clock cycles for the pulse
            %       to remain high for. A clock cycle corresponds to ~5ns.
            %
            %See Also PORT_DRIVE, PORT_READ, PORT_PWM
            if obj.device_pointer == 0
                error('Device needs to be opened.')
            end
            
            port_drive_gw(obj.device_pointer, port, threshold);
        end
        
        function port_pwm(obj, port, threshold, period)
            %PORT_PWM Pulse width modulate a DPD80 port.
            %
            %SYNTAX: device.port_pwm(port, threshold, period)
            %
            %INPUTS:
            %       port (string): The port that you want to pulse width modulate. Acceptable values
            %       are 'A' or 'B'. Please note that port A is low
            %       passed at ~1 KHz, so when it is pulse width modulated it creates an analog output with a rise time of ~ 1ms.
            %
            %       threshold (int): The number of clock cycles in each
            %       period that the signal should remain high for. Each
            %       clock cycle is ~5 ns.
            %
            %       period (int): The number of clock cycles each pwm
            %       cycle. Period must be greater than threshold. Each
            %       clock cycle is ~5 ns.
            %
            %See Also PORT_DRIVE, PORT_PULSE, PORT_READ
            if obj.device_pointer == 0
                error('Device needs to be opened.')
            end
            
            port_drive_gw(obj.device_pointer, port, threshold, period);
        end
        
        function val = port_read(obj, port)
            %PORT_READ Digitally reads the voltage at the given port.
            %
            %SYNTAX: val = device.read_port(port)
            %
            %INPUTS:
            %       port (string): The port that you want to pulse width modulate. Acceptable values
            %       are 'A' or 'B'. Please note that port A is low
            %       passed at ~1 KHz, so when it is pulse width modulated it creates an analog output with a rise time of ~ 1ms.
            %
            %OUTPUTS:
            %       val (int): Return 1 if the voltage of the port is
            %       greater than 2.15 V and  0 of the voltage is less than
            %       0.82 V.
            %
            %See Also PORT_DRIVE, PORT_PULSE, PORT_PWM
            %
            val = port_read_gw(obj.device_pointer, port);
        end
        
        function val = read_highgain(obj)
            %READ_HIGHGAIN Read the gain mode.
            %
            %SYNTAX: val = device.read_highgain()
            %
            %OUTPUTS:
            %       val (int): Return 1 if the device is currently in
            %       highgain mode and 0 if the device is in low gain mode
            %
            %See Also SET_HIGHGAIN
            %
            if obj.device_pointer == 0
                error('Device needs to be opened.')
            end
            
            val = read_highgain_gw(obj.device_pointer);
        end
        
        function val = read_antialias(obj)
            %READ_ANTIALIAS Read the antialiasing filter's state
            %
            %SYNTAX: val = device.read_antialias()
            %
            %OUTPUTS:
            %       val (int): Return 1 if the antialiasing fiter is on and
            %       0 if the antialiasing filter is off.
            %
            %See Also SET_ANTALIAS
            %
            val = read_antialias_gw(obj.device_pointer);
        end
        
        function set_antialias(obj, val)
            %SET_ANTIALIAS Turn on or off the antialiasing filter
            %
            %SYNTAX: device.set_antialias(val)
            %
            %INPUTS:
            %       val (boolean): If val is true it will turn on the
            %       filter and if it is fals it will turn off the filter
            %
            %See Also READ_ANTIALIAS
            %
            if obj.device_pointer == 0
                error('Device needs to be opened.')
            end
            
            set_antialias_gw(obj.device_pointer, val);
        end
        
        function set_highgain(obj, val)
            %SET_HIGHGAIN Set the device to either high or low gain mode
            %
            %SYNTAX: device.set_highgain(val)
            %
            %INPUTS:
            %       val (boolean): If val is true it will turn on high gain
            %       mode and if it is false it will turn on low gain mode.
            %
            %See Also READ_HIGHGAIN
            %
            set_antialias_gw(obj.device_pointer, val);
        end
        
        function obj = close(obj)
            %CLOSE Closes the connection to the DPD80
            %
            %SYNTAX: device.close()
            %
            if obj.device_pointer == 0
                error('Device needs to be opened.')
            end
            
            close_gw(obj.device_pointer);
            obj.device_pointer = 0;
            obj.opened = false;
        end
    end
end


